@@ -20,6 +20,9 @@
 #include "ubuntushared.h"
 #include "ubuntuconstants.h"
 #include "ubuntudeviceswidget.h"
+#include "ubuntuproject.h"
+#include "ubuntuclicktool.h"
+#include "ubuntudevice.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/icontext.h>
@@ -27,13 +30,16 @@
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/messagemanager.h>
-#include <qt4projectmanager/qt4projectmanagerconstants.h>
+#include <qmakeprojectmanager/qmakeprojectmanagerconstants.h>
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/session.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/compileoutputwindow.h>
 #include <projectexplorer/iprojectmanager.h>
+#include <projectexplorer/buildconfiguration.h>
 #include <utils/qtcprocess.h>
+#include <ssh/sshconnection.h>
 
 #include <QProcessEnvironment>
 #include <QAction>
@@ -48,13 +54,30 @@
 using namespace Ubuntu;
 using namespace Ubuntu::Internal;
 
+UbuntuMenu *UbuntuMenu::m_instance = 0;
+
+UbuntuMenu *UbuntuMenu::instance()
+{
+    return m_instance;
+}
+
+QAction *UbuntuMenu::menuAction(const Core::Id &id)
+{
+    UbuntuMenu* men = UbuntuMenu::instance();
+    if(men->m_actions.contains(id))
+        return men->m_actions[id];
+    return 0;
+}
+
 UbuntuMenu::UbuntuMenu(QObject *parent) :
     QObject(parent)
 {
+    m_instance = this;
     m_obj = getMenuJSON();
 
     connect(&m_ubuntuProcess,SIGNAL(message(QString)),this,SLOT(onMessage(QString)));
     connect(&m_ubuntuProcess,SIGNAL(finished(QString,int)),this,SLOT(onFinished(QString,int)));
+    connect(&m_ubuntuProcess,SIGNAL(finished(const QProcess*,QString,int)),this,SLOT(onFinished(const QProcess*,QString,int)));
     connect(&m_ubuntuProcess,SIGNAL(started(QString)),this,SLOT(onStarted(QString)));
     connect(&m_ubuntuProcess,SIGNAL(error(QString)),this,SLOT(onError(QString)));
 
@@ -64,18 +87,18 @@ UbuntuMenu::UbuntuMenu(QObject *parent) :
 
 
 void UbuntuMenu::slotUpdateActions() {
-    ProjectExplorer::ProjectExplorerPlugin* projectExplorerInstance = ProjectExplorer::ProjectExplorerPlugin::instance();
-    ProjectExplorer::Project* startupProject = projectExplorerInstance->startupProject();
+    ProjectExplorer::Project* startupProject = ProjectExplorer::SessionManager::startupProject();
     bool isQmlProject = false;
     bool isQmakeProject = false;
     bool isUbuntuProject = false;
-    bool isCordovaProject = false;
+    bool isUbuntuHtmlProject = false;
 
     if (startupProject) {
-        isQmlProject = (startupProject->projectManager()->mimeType() == QLatin1String("application/x-qmlproject"));
-        isQmakeProject = (startupProject->projectManager()->mimeType() == QLatin1String("application/vnd.nokia.qt.qmakeprofile"));
-        isCordovaProject = (startupProject->projectManager()->mimeType() == QLatin1String("application/x-cordovaproject"));
+        isQmlProject = (startupProject->projectManager()->mimeType() == QLatin1String(Constants::QMLPROJECT_MIMETYPE));
+        isQmakeProject = (startupProject->projectManager()->mimeType() == QLatin1String(Constants::QMAKE_MIMETYPE));
         isUbuntuProject = (startupProject->projectManager()->mimeType() == QLatin1String(Constants::UBUNTUPROJECT_MIMETYPE));
+        isUbuntuHtmlProject = isProperUbuntuHtmlProject(startupProject);
+        isUbuntuProject = isUbuntuProject || isUbuntuHtmlProject || isQmlProject;
     }
 
     //bool canRun = projectExplorerInstance->canRun(startupProject,ProjectExplorer::NormalRunMode);
@@ -86,17 +109,22 @@ void UbuntuMenu::slotUpdateActions() {
         bool requiresDevice = act->property(Constants::UBUNTU_MENUJSON_DEVICEREQUIRED).toBool();
         bool requiresProject = act->property(Constants::UBUNTU_MENUJSON_PROJECTREQUIRED).toBool();
         bool requiresQmlProject = act->property(Constants::UBUNTU_MENUJSON_QMLPROJECTREQUIRED).toBool();
-	bool requiresCordovaProject = act->property(Constants::UBUNTU_MENUJSON_CORDOVAPROJECTREQUIRED).toBool();
         bool requiresQmakeProject = act->property(Constants::UBUNTU_MENUJSON_QMAKEPROJECTREQUIRED).toBool();
         bool requiresUbuntuProject = act->property(Constants::UBUNTU_MENUJSON_UBUNTUPROJECTREQUIRED).toBool();
-        bool actionEnabled = ( (requiresQmakeProject ? isQmakeProject : true) && (requiresQmlProject ? isQmlProject : true) && (requiresDevice ? deviceDetected : true) && (requiresProject ? projectOpen : true) && (requiresUbuntuProject ? isUbuntuProject : true) && (requiresCordovaProject ? isCordovaProject : true));
+        bool requiresUbuntuHtmlProject = act->property(Constants::UBUNTU_MENUJSON_UBUNTUHTMLPROJECTREQUIRED).toBool();
+        bool actionEnabled = ( (requiresQmakeProject ? isQmakeProject : true) &&
+                               (requiresQmlProject ? isQmlProject : true) &&
+                               (requiresUbuntuHtmlProject ? isUbuntuHtmlProject : true) &&
+                               (requiresDevice ? deviceDetected : true) &&
+                               (requiresProject ? projectOpen : true) &&
+                               (requiresUbuntuProject ? isUbuntuProject : true));
 
         act->setEnabled( actionEnabled );
     }
 }
 
 void UbuntuMenu::onStarted(QString cmd) {
-    printToOutputPane(QString::fromLatin1("Started %0").arg(cmd));
+    printToOutputPane(QString::fromLatin1(Constants::UBUNTUMENU_ONSTARTED).arg(cmd));
 }
 
 void UbuntuMenu::onMessage(QString msg) {
@@ -104,11 +132,17 @@ void UbuntuMenu::onMessage(QString msg) {
 }
 
 void UbuntuMenu::onError(QString msg) {
-    printToOutputPane(QString::fromLatin1("%0").arg(msg));
+    printToOutputPane(QString::fromLatin1(Constants::UBUNTUMENU_ONERROR).arg(msg));
 }
 
 void UbuntuMenu::onFinished(QString cmd, int code) {
-    printToOutputPane(QString::fromLatin1("%0 finished with code %1").arg(cmd).arg(code));
+    emit finished_action(cmd);
+    printToOutputPane(QString::fromLatin1(Constants::UBUNTUMENU_ONFINISHED).arg(cmd).arg(code));
+}
+
+void UbuntuMenu::onFinished(const QProcess *programm, QString cmd, int)
+{
+    emit finished_action(programm,cmd);
 }
 
 QString UbuntuMenu::menuPath(QString fileName) {
@@ -138,7 +172,7 @@ void UbuntuMenu::parseMenu(QJsonObject obj, Core::ActionContainer*& parent, cons
         if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_ID))) {
             menuId = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_ID)).toString();
         }
-        Core::ActionContainer *actionContainer = Core::ActionManager::createMenu(Core::Id(menuId));
+        Core::ActionContainer *actionContainer = Core::ActionManager::createMenu(Core::Id(menuId.toUtf8().constData()));
         actionContainer->menu()->setTitle(menuName);
         actionContainer->menu()->setObjectName(menuId);
 
@@ -163,12 +197,15 @@ void UbuntuMenu::parseMenu(QJsonObject obj, Core::ActionContainer*& parent, cons
         QString actionId;
         QString actionKeySequence;
         QString actionWorkingDirectory;
+        QStringList contexts = QStringList()<<QLatin1String(Core::Constants::C_GLOBAL);
         bool actionProjectRequired = false;
         bool actionDeviceRequired = false;
         bool actionQmlProjectRequired = false;
         bool actionQmakeProjectRequired = false;
         bool actionUbuntuProjectRequired = false;
+        bool actionUbuntuHtmlProjectRequired = false;
         bool actionSaveRequired = false;
+        bool clickTargetRequired = false;
 
         if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_NAME))) {
             actionName = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_NAME)).toString();
@@ -194,17 +231,38 @@ void UbuntuMenu::parseMenu(QJsonObject obj, Core::ActionContainer*& parent, cons
         if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_QMAKEPROJECTREQUIRED))) {
             actionQmakeProjectRequired = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_QMAKEPROJECTREQUIRED)).toBool();
         }
-
         if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_UBUNTUPROJECTREQUIRED))) {
             actionUbuntuProjectRequired = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_UBUNTUPROJECTREQUIRED)).toBool();
         }
-
+        if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_UBUNTUHTMLPROJECTREQUIRED))) {
+            actionUbuntuHtmlProjectRequired = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_UBUNTUHTMLPROJECTREQUIRED)).toBool();
+        }
         if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_SAVEREQUIRED))) {
             actionSaveRequired = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_SAVEREQUIRED)).toBool();
         }
+        if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_CONTEXT))) {
+            //contexts can contains either a string, or a array of strings
+            QJsonValue v = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_CONTEXT));
+            if(v.isArray()) {
+                QJsonArray jsonContexts = v.toArray();
+
+                QStringList tmp_contexts;
+                for (int i = 0; i < jsonContexts.size(); i++) {
+                    tmp_contexts.append(jsonContexts.at(i).toString());
+                }
+
+                if(!tmp_contexts.isEmpty())
+                    contexts = tmp_contexts;
+
+            } else {
+                contexts = QStringList()<<obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_CONTEXT)).toString();
+            }
+        }
+        if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_CLICKTARGETREQUIRED))) {
+            clickTargetRequired = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_CLICKTARGETREQUIRED)).toBool();
+        }
 
         if (obj.contains(QLatin1String(Constants::UBUNTU_MENUJSON_ACTIONS)) && obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_ACTIONS)).isArray()) {
-
             QJsonValue actions = obj.value(QLatin1String(Constants::UBUNTU_MENUJSON_ACTIONS));
             if (actions.isArray()) {
                 QJsonArray actionsArray = actions.toArray();
@@ -213,40 +271,58 @@ void UbuntuMenu::parseMenu(QJsonObject obj, Core::ActionContainer*& parent, cons
                     QJsonValue actionItem = actionsArray.at(idx);
                     actionsList.append(actionItem);
                 }
-                m_commandMap.insert(actionName,actionsList);
+                m_commandMap.insert(actionId,actionsList);
             }
         }
 
         QAction *act= new QAction(actionName, this);
         act->setObjectName(actionId);
-
-        Core::Command *cmd = Core::ActionManager::registerAction(act, Core::Id(actionId), Core::Context(Core::Constants::C_GLOBAL));
-        if (actionKeySequence.isEmpty() == false) {
-            cmd->setDefaultKeySequence(QKeySequence(actionKeySequence));
-        }
-        if (actionWorkingDirectory.isEmpty() == false) {
-            act->setProperty(Constants::UBUNTU_MENUJSON_WORKINGDIRECTORY,actionWorkingDirectory);
-        }
-
         act->setProperty(Constants::UBUNTU_MENUJSON_PROJECTREQUIRED,actionProjectRequired);
         act->setProperty(Constants::UBUNTU_MENUJSON_DEVICEREQUIRED,actionDeviceRequired);
         act->setProperty(Constants::UBUNTU_MENUJSON_QMAKEPROJECTREQUIRED,actionQmakeProjectRequired);
         act->setProperty(Constants::UBUNTU_MENUJSON_QMLPROJECTREQUIRED,actionQmlProjectRequired);
         act->setProperty(Constants::UBUNTU_MENUJSON_UBUNTUPROJECTREQUIRED,actionUbuntuProjectRequired);
+        act->setProperty(Constants::UBUNTU_MENUJSON_UBUNTUHTMLPROJECTREQUIRED,actionUbuntuHtmlProjectRequired);
         act->setProperty(Constants::UBUNTU_MENUJSON_SAVEREQUIRED,actionSaveRequired);
+        act->setProperty(Constants::UBUNTU_MENUJSON_CLICKTARGETREQUIRED,clickTargetRequired);
 
         connect(act, SIGNAL(triggered()), this, SLOT(menuItemTriggered()));
-        m_actions.append(act);
+        m_actions.insert(Core::Id(actionId.toUtf8().constData()),act);
 
-        if (parent == NULL) {
-            qWarning() << "No menu defined";
-        } else {
-            parent->addAction(cmd,group);
+        foreach(const QString& context,contexts) {
+            Core::Command *cmd = Core::ActionManager::registerAction(act, Core::Id(actionId.toUtf8().constData()), Core::Context(context.toUtf8().constData()));
+            if (actionKeySequence.isEmpty() == false) {
+                cmd->setDefaultKeySequence(QKeySequence(actionKeySequence));
+            }
+            if (actionWorkingDirectory.isEmpty() == false) {
+                act->setProperty(Constants::UBUNTU_MENUJSON_WORKINGDIRECTORY,actionWorkingDirectory);
+            }
+
+            //hide if the context does not match creators current context
+            cmd->setAttribute(Core::Command::CA_Hide);
+
+            if (parent == NULL) {
+                qWarning() << Constants::ERROR_MSG_NO_MENU_DEFINED;
+            } else {
+                parent->addAction(cmd,group);
+            }
         }
     }
 
 }
 
+bool UbuntuMenu::isProperUbuntuHtmlProject(ProjectExplorer::Project *project) const
+{
+    if (Q_UNLIKELY(NULL == project))
+        return false;
+
+    UbuntuProject* ubuntuProject = qobject_cast<UbuntuProject*>(project);
+    if (NULL == ubuntuProject)
+        return false;
+
+    return ubuntuProject->filesFileName().endsWith(QLatin1String(Constants::UBUNTUHTMLPROJECT_SUFFIX));
+}
+
 void UbuntuMenu::menuItemTriggered() {
     QAction* act = qobject_cast<QAction*>(sender());
     if (act) {
@@ -260,7 +336,7 @@ void UbuntuMenu::menuItemTriggered() {
         ProjectExplorer::Project* project = NULL;
 
         if (projectRequired.isValid() && projectRequired.toBool() == true) {
-            project = ProjectExplorer::ProjectExplorerPlugin::instance()->startupProject();
+            project = ProjectExplorer::SessionManager::startupProject();
 
             if (project == NULL) {
                 QMessageBox::information(Core::ICore::mainWindow(),QLatin1String(Constants::UBUNTU_DIALOG_NO_PROJECTOPEN_TITLE),QLatin1String(Constants::UBUNTU_DIALOG_NO_PROJECTOPEN_TEXT));
@@ -273,8 +349,8 @@ void UbuntuMenu::menuItemTriggered() {
             ProjectExplorer::ProjectExplorerPlugin::instance()->saveModifiedFiles();
         }
 
-        if (m_commandMap.contains(act->text())) {
-            QJsonValueList actions = m_commandMap.value(act->text());
+        if (m_commandMap.contains(act->objectName())) {
+            QJsonValueList actions = m_commandMap.value(act->objectName());
 
             QString queryData;
             bool bQueryOk = false;
@@ -349,7 +425,11 @@ void UbuntuMenu::menuItemTriggered() {
 
                         QString projectDirectory = project->projectDirectory();
                         QString displayName = project->displayName();
-                        QString folderName = QFileInfo(projectDirectory).baseName();
+
+                        QString folderName = projectDirectory;
+                        // Bug 1212937 workaround
+                        folderName = folderName.replace(QString(QLatin1String("%0/")).arg(QFileInfo(projectDirectory).path()),QLatin1String(""));
+
                         QStringList projectFiles = project->files(ProjectExplorer::Project::AllFiles);
 
                         QString workingDirectoryData = act->property(Constants::UBUNTU_MENUJSON_WORKINGDIRECTORY).toString();
@@ -358,16 +438,53 @@ void UbuntuMenu::menuItemTriggered() {
                         } else {
                             workingDirectory = projectDirectory;
                         }
+
                         command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_PROJECTDIRECTORY),projectDirectory);
                         command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_FOLDERNAME),folderName);
                         command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_DISPLAYNAME),displayName);
                         command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_PROJECTFILES),projectFiles.join(QLatin1String(" ")));
+
+                        if (isProperUbuntuHtmlProject(project)) {
+                            command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_APP_RUNNER_EXECNAME),
+                                                      QLatin1String(Constants::UBUNTUHTML_PROJECT_LAUNCHER_EXE));
+                        }
+
+                        QVariant clickTargetRequired = act->property(Constants::UBUNTU_MENUJSON_CLICKTARGETREQUIRED);
+                        if(clickTargetRequired.isValid() && clickTargetRequired.toBool()) {
+                            UbuntuClickTool::Target clickTarget;
+                            if(!UbuntuClickTool::getTargetFromUser(&clickTarget))
+                                return;
+
+                            command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_CLICK_PACKAGING_FOLDER)
+                                                      ,QString::fromLatin1(Constants::UBUNTU_CLICK_CLICK_PACKAGE_DIR));
+                            command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_CLICK_ARCH),clickTarget.architecture);
+                            command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_CLICK_FRAMEWORK),clickTarget.framework);
+                            command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_CLICK_SERIES),clickTarget.series);
+
+                            //this is a clicktarget, so we change the builddirectory to the current active buildconfig
+                            //directory
+                            ProjectExplorer::Target* qtcTarget = project->activeTarget();
+                            if(!qtcTarget)
+                                return;
+
+                            ProjectExplorer::BuildConfiguration* qtcBuildConfig = qtcTarget->activeBuildConfiguration();
+                            if(!qtcBuildConfig)
+                                return;
+
+                            workingDirectory = qtcBuildConfig->buildDirectory().toString();
+                        }
                     }
 
+                    UbuntuDevice::ConstPtr device = UbuntuDevicesWidget::instance()->device();
+                    if (device) {
+                        command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_DEVICE_IP),device->sshParameters().host);
+                        command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_DEVICE_USERNAME),device->sshParameters().userName);
+                        command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_DEVICE_PORT),QString::number(device->sshParameters().port));
+
+                        command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_SERIALNUMBER),device->id().toSetting().toString());
+                    }
                     command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_SHAREDIRECTORY),Constants::UBUNTU_SHAREPATH);
                     command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_SCRIPTDIRECTORY),Constants::UBUNTU_SCRIPTPATH);
-                    command = command.replace(QLatin1String(Constants::UBUNTU_ACTION_SERIALNUMBER),UbuntuDevicesWidget::instance()->serialNumber());
-
                     if (bQuery && bQueryOk) {
                         command = QString(command).arg(queryData);
                     }
@@ -379,10 +496,10 @@ void UbuntuMenu::menuItemTriggered() {
             }
             m_ubuntuProcess.start(act->text());
         } else {
-            qWarning() << __PRETTY_FUNCTION__  << "No actions defined in map";
+            qWarning() << __PRETTY_FUNCTION__  << Constants::ERROR_MSG_NOACTIONS;
         }
     } else {
-        qWarning() << __PRETTY_FUNCTION__  << "Could not cast to action";
+        qWarning() << __PRETTY_FUNCTION__  << Constants::ERROR_MSG_COULD_NOT_CAST_TO_ACTION;
     }
 }
 
@@ -405,7 +522,7 @@ void UbuntuMenu::initialize() {
                     else if (parentValue == QLatin1String(Constants::UBUNTU_MENUJSON_PARENT_FILE)) actionContainer = Core::ActionManager::actionContainer(Core::Constants::M_FILE);
                     else if (parentValue == QLatin1String(Constants::UBUNTU_MENUJSON_PARENT_BUILD)) actionContainer = Core::ActionManager::actionContainer(ProjectExplorer::Constants::M_BUILDPROJECT);
                     else if (parentValue == QLatin1String(Constants::UBUNTU_MENUJSON_PARENT_TOP)) actionContainer = Core::ActionManager::actionContainer(Core::Constants::MENU_BAR);
-                    else actionContainer = Core::ActionManager::actionContainer(Core::Id(parentValue));
+                    else actionContainer = Core::ActionManager::actionContainer(Core::Id(parentValue.toUtf8().constData()));
                 } else {
                     actionContainer = Core::ActionManager::actionContainer(Core::Constants::M_TOOLS);
                 }
@@ -417,12 +534,12 @@ void UbuntuMenu::initialize() {
 
                 Core::Id groupId;
                 if (!group.isEmpty()) {
-                    groupId = Core::Id(group);
+                    groupId = Core::Id(group.toUtf8().constData());
                 }
                 parseMenu(obj,actionContainer,groupId);
             }
         }
     } else {
-        qDebug() << "json is not valid";
+        qWarning() << Constants::ERROR_MSG_UNABLE_TO_PARSE_MENUJSON;
     }
 }
