@@ -27,6 +27,8 @@
 namespace Ubuntu {
 namespace Internal {
 
+class UbuntuClickManifest;
+
 class UbuntuPackagingMode : public Core::IMode
 {
     Q_OBJECT
@@ -34,17 +36,24 @@ public:
     explicit UbuntuPackagingMode(QObject *parent = 0);
     void initialize();
 
+    static UbuntuClickManifest *manifest ();
+    static UbuntuClickManifest *appArmor ();
+
 protected slots:
     void modeChanged(Core::IMode*);
 
     void on_projectAdded(ProjectExplorer::Project *project);
     void on_projectRemoved(ProjectExplorer::Project *project);
+    void updateModeState();
 
 protected:
     QWidget* m_modeWidget;
     UbuntuPackagingWidget m_ubuntuPackagingWidget;
 
     Core::Id previousMode;
+
+private:
+    static UbuntuPackagingMode* m_instance;
 };
 
 }
